#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# 用于替代依赖安装脚本
# newos 在up_post阶段执行，仅一次
# base 在后台运行时执行

import subprocess
import os
import logging

DEPENDS_PATH="/usr/share/revisions-manager/data/depends/"


openkylin_base_pkg_dict={
    "kylin-daq":"1.2.9.1-0k2.1openkylin1.5c",
    "aria2":"1.35.0-ok1",
    "libaria2-0":"1.35.0-ok1",
    "libc-ares2":"1.15.0-ok1",
}

openkylin_newos_pkg_dict={
    "bubblewrap":"0.4.0-ok3",
    "containerd":"1.5.9-ok6",
    "debootstrap":"1.0.118-ok4",
    "docker.io":"20.10.16-ok1",
    "runc":"1.1.0-ok1",
    "kare":"2.2.4.7",   
    "kylin-daq":"1.2.9.1-0k2.1openkylin1.5c",
    "aria2":"1.35.0-ok1",
    "libaria2-0":"1.35.0-ok1",
    "libc-ares2":"1.15.0-ok1",
}

# 升级功能需要, 在 oldos 中安装
kylin_base_pkg_dict={
    "librockey":"1.0-1.11kord",
    "kylin-daq":"1.2.9.1-0k2.6.2",
    "qttranslations5-l10n":"5.6.1-2kord5",   # 修改 v10 的翻译
    "libssl1.0.0":"1.0.2g-1kord4.20+esm11",  # kylin-daq
    "aria2":"1.19.0-1kord1",
    "libnettle6":"3.2-1kord0.16.04.1",
    "libc-ares2":"1.10.0-3kord0.2",
    "kylin-activation":"1.3.11kord",
    "libkylin-activation":"3.0-64kord",
}

# 新系统中功能需要, 在 newos 中安装
kylin_newos_pkg_dict={
    "bubblewrap":"0.4.0-1kylin4",           # kare
    "containerd":"1.6.12-0kylin1~20.04.3",  # kare
    "debootstrap":"1.0.118kylin2",          # kare
    "docker.io":"20.10.7-0kylin5~20.04.2",  # kare
    "runc":"1.1.4-0kylin1~20.04.3",         # kare
    "kare":"2.3.1.1",                      # kare
    "kylin-daq":"1.2.9.1-0k2.6.2",          # kylin-daq
    "libssl1.0.0":"1.0.2g-1kord4.20+esm11", # kylin-daq
    "aria2":"1.19.0-1kord1",                # aria2
    "libnettle6":"3.2-1kord0.16.04.1",      # aria2
    "libc-ares2":"1.10.0-3kord0.2",         # aria2
}
# 新系统中修改问题, 在2303中安装
kylin_newos_pkg_dict_2303={
    "kylin-installer":"2.0.2.1-0k2.18.3",                   # 修改2303 卸载兼容应用会卸载bash的问题
    "kylin-software-properties":"0.0.1.1-0k0.11update1",    # 修改2303 过多的 ping 
}
# 新系统中修改问题, 替换相同版本
kylin_newos_pkg_dict_ex={
    "kylin-software-center-plugin-synchrodata":"5.0.6.8-0k0.29", # 修改 2303 core
}

class RMDepends():
    def __init__(self) -> None:
        self.arch = ""
        self.debPath = ""
        self._update_self()

    def _update_self(self):
        cmd = "dpkg-architecture -q DEB_TARGET_ARCH"
        ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode != 0:
            logging.error("[RMDEP]: [%s] err [%s].", cmd, ret.stderr.decode().strip("\n"))
        else:
            self.arch = ret.stdout.decode().strip("\n")
            if self.arch == "mips64":
                self.debPath = DEPENDS_PATH + "mips64el/"
            else:
                self.debPath = DEPENDS_PATH + self.arch + "/"

    def is_deb_exist(self, debName):
        eleList = os.listdir(self.debPath)
        for ele in eleList:
            if debName in ele:
                return True
        return False

    def _install_deb_dict(self, debDict):
        if not os.path.exists(self.debPath):
            logging.error("[RMDEP]: path[%s] not exists.", self.debPath)
            return -1
        if len(debDict) == 0:
            logging.info("[RMDEP]: No deb")
            return 0

        cmdStr = "apt install -y "
        path = "/usr/share/revisions-manager/data/apt_private.conf"
        if os.path.exists(path):
            cmdStr = "apt install -y -c /usr/share/revisions-manager/data/apt_private.conf "
        needInstall = False
        for ele in debDict:
            debVer = debDict[ele]
            logging.info("[RMDEP]: check pkg[%s] ", ele)
            cmd = "dpkg-query --show " + ele + " | awk \'{print $2}\'"
            ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if ret.returncode != 0:
                logging.error("[RMDEP]: [%s] err [%s].", cmd, ret.stderr.decode().strip("\n"))
                continue
            curVer = ret.stdout.decode().strip("\n")
            # 版本低 或 不存在 -> 安装
            if len(curVer) > 0:
                cmd = "dpkg --compare-versions " + curVer + " ge " + debVer
                ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                if ret.returncode == 0:
                    logging.info("[RMDEP]: [%s][%s] exists.", ele, curVer)
                    continue
            if not self.is_deb_exist(ele):
                logging.warning("[RMDEP]: [%s] not exist.", ele)
                continue
            needInstall = True
            cmdStr = cmdStr + self.debPath + ele + "_" + debVer + "*.deb "

        if not needInstall:
            logging.info("[RMDEP]: Not install deb")
            return 0

        self._disable_signstatus()

        cmdStr = cmdStr + " >> /var/log/RevisionsManager/depends-install.log"
        ret = subprocess.run(cmdStr, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode != 0:
            self._enable_signstatus()
            logging.error("[RMDEP]: [%s] install depends [%s]", cmdStr, ret.stderr.decode().strip("\n"))
            return -2
        else:
            self._enable_signstatus()
            logging.info("[RMDEP]: [%s] install depends [%s]", cmdStr, ret.stdout.decode().strip("\n"))
            return 0

    def _install_deb_dict_ex(self, debDict): 
        if not os.path.exists(self.debPath):
            logging.error("[RMDEP]:EX path[%s] not exists.", self.debPath)
            return -1
        if len(debDict) == 0:
            logging.info("[RMDEP]:EX No deb")
            return 0

        cmdStr = "apt install -y "
        needInstall = False
        for ele in debDict:
            debVer = debDict[ele]
            logging.info("[RMDEP]:EX check pkg[%s] ", ele)
            cmd = "dpkg-query --show " + ele + " | awk \'{print $2}\'"
            ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if ret.returncode != 0:
                logging.error("[RMDEP]:EX [%s] err [%s].", cmd, ret.stderr.decode().strip("\n"))
                continue
            curVer = ret.stdout.decode().strip("\n")
            # 不存在 或 版本不同 -> 不安装
            if len(curVer) == 0 or curVer != debVer:
                logging.info("[RMDEP]:EX [%s][%s] not need.", ele, debVer)
                continue
            if not self.is_deb_exist(ele):
                logging.warning("[RMDEP]:EX [%s] not exist.", ele)
                continue
            needInstall = True
            cmdStr = cmdStr + self.debPath + ele + "_" + debVer + "*.deb "
        if not needInstall:
            logging.info("[RMDEP]:EX Not install deb")
            return 0

        self._disable_signstatus()

        cmdStr = cmdStr + " >> /var/log/RevisionsManager/depends-install.log"
        ret = subprocess.run(cmdStr, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode != 0:
            logging.error("[RMDEP]:EX [%s] install depends [%s]", cmdStr, ret.stderr.decode().strip("\n"))
        else:
            logging.info("[RMDEP]:EX [%s] install depends [%s]", cmdStr, ret.stdout.decode().strip("\n"))
        
        self._enable_signstatus()

    def install_base_kylin(self):
        self._install_deb_dict(kylin_base_pkg_dict)

    def install_newos_kylin(self):
        ret = self._install_deb_dict(kylin_newos_pkg_dict)
        if ret == -2:
            self._install_deb_dict(kylin_newos_pkg_dict)

    def install_base_openkylin(self):
        self._install_deb_dict(openkylin_base_pkg_dict)

    def install_newos_openkylin(self):
        self._install_deb_dict(openkylin_newos_pkg_dict)

    def install_newos_2303(self):
        self._install_deb_dict(kylin_newos_pkg_dict_2303)

    def install_newos_kylin_ex(self):
        self._install_deb_dict_ex(kylin_newos_pkg_dict_ex)

    def _disable_signstatus(self):
        try:
            cmd = "setsignstatus off"
            ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if ret.returncode != 0:
                logging.error("[RMDEP]: [%s] out[%s] err[%s].", cmd, ret.stdout.decode().strip("\n"), ret.stderr.decode().strip("\n"))
        except Exception as e:
            logging.error("[RMDEP]: disable sign err[%s]", e)

    def _enable_signstatus(self):
        try:
            cmd = "setsignstatus warning"
            ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if ret.returncode != 0:
                logging.error("[RMDEP]: [%s] out[%s] err[%s].", cmd, ret.stdout.decode().strip("\n"), ret.stderr.decode().strip("\n"))
        except Exception as e:
            logging.error("[RMDEP]: enable sign err[%s]", e)

if __name__ == "__main__":
    handler = RMDepends()
    handler.install_base_openkylin()

