#!/usr/bin/python3

import logging
import os

OSRELEASE_PATH = "/etc/os-release"
RMSUCCESS = 0
RMFAILURE = -1


class RMOsInfo():
    VERSION_KYLIN_V10 = 0
    VERSION_KYLIN_V10SP1 = 1
    VERSION_OPENKYLIN = 2
    VERSION_UKYLIN = 3
    VERSION_UNKNOWN = 4

    def __init__(self, osReleasePath = OSRELEASE_PATH):
        self.osName = ""
        self.prettyName = ""
        self.proCodeName = ""
        self.versionId = ""
        self.releaseId = ""
        self.osNum = self.VERSION_KYLIN_V10

        self._load_os_release(osReleasePath)
        self._update_os_num()

    def _load_os_release(self, osReleasePath):
        if not os.path.exists(osReleasePath):
            logging.error("[OSINFO]: os-release not exists.")
            return RMFAILURE
        infoList = []
        with open(osReleasePath, "r") as fd:
            infoList = fd.readlines()

        if len(infoList) == 0:
            logging.error("[OSINFO]: no info in os-release.")
            return RMFAILURE
        
        for ele in infoList:
            try:
                tmpList = ele.strip("\n").split("=")
                if len(tmpList) != 2:
                    logging.info("[OSINFO]: info[%s] is not standard.", ele.strip("\n"))
                    continue
                value = tmpList[1].strip("\"")
                if tmpList[0] == "NAME":
                    self.osName = value
                elif tmpList[0] == "PRETTY_NAME":
                    self.prettyName = value
                elif tmpList[0] == "PROJECT_CODENAME":
                    self.proCodeName = value
                elif tmpList[0] == "VERSION_ID":
                    self.versionId = value
                elif tmpList[0] == "KYLIN_RELEASE_ID":
                    self.releaseId = value
            except Exception as e:
                logging.error("[OSINFO]: info [%s] err [%s].", ele.strip("\n"), e)
        
        return RMSUCCESS
    
    def _update_os_num(self):
        if self.osName.lower() == "openkylin":
            self.osNum = self.VERSION_OPENKYLIN
        elif self.osName.lower() == "ubuntu kylin":
            self.osNum = self.VERSION_UKYLIN
        elif self.osName.lower() == "kylin":
            if self.prettyName.lower() == "kylin v10 sp1":
                self.osNum = self.VERSION_KYLIN_V10SP1
            elif self.prettyName.lower() == "kylin v10":
                self.osNum = self.VERSION_KYLIN_V10
        logging.info("[OSINFO]: os name is [%s].", self.osName)

    def get_pretty_name(self):
        return self.prettyName
    def get_os_name(self):
        return self.osName
    def get_version_id(self):
        return self.versionId
    def get_os_num(self):
        return self.osNum
    def get_release_id(self):
        return self.releaseId


if __name__ == "__main__":

    osInfo = RMOsInfo()
    osInfo.get_os_name()

