# 从服务端获取模块
# 下载映射表
# 根据映射表下载显示配置、种子文件、授权文件、镜像
# TODO: 当前支持1个ISO下载, 下载多个并有其他取消操作时, 可能会有问题

import re
import os
import dbus
import json
import logging
import threading
import subprocess
from time import sleep
from xmlrpc import client
from gettext import gettext as _
from .RMOsInfo import RMOsInfo
from .RMConf import RMConf
from .RMCom import RMCom, get_sys_call, get_download_path


DEFAULT_REMOTE_ADDRESS = "https://releases.openkylin.top/upgrade-config/"
REMOTE_STATE_PATH = RMCom.CACHE_DIR + 'kylin-revision-manager-remote.state' #记录本模块相关的状态
DOWNLOAD_ISO_INFO_PATH = RMCom.CACHE_DIR + 'download-iso-info'  # 记录下载的镜像相关信息, 同时用于升级后对原来下载的ISO做种
SOURCES_INFO_PATH = RMCom.CACHE_DIR + 'this-sources-info'  # 记录匹配的源相关信息，用于升级过程中和升级成功后的授权文件下载
REMOTE_DOWNLOAD_DIR = RMCom.CACHE_DIR + "download/"
LOG_PATH = REMOTE_DOWNLOAD_DIR + "wget_log"
SOURCES_FILE = "kylin-revision-manager-sources.json"
DISPLY_FILE = "kylin-revision-manager-display.json"
LICENSE_FILE = 'LICENSE'
KYINFO_FILE = '.kyinfo'
DL_SOURCES_PATH = REMOTE_DOWNLOAD_DIR + SOURCES_FILE
DL_DISPLAY_PATH = REMOTE_DOWNLOAD_DIR + DISPLY_FILE
DL_LICENSE_PATH = REMOTE_DOWNLOAD_DIR + LICENSE_FILE
DL_KYINFO_PATH = REMOTE_DOWNLOAD_DIR + KYINFO_FILE

KYINFO_PATH = "/etc/.kyinfo"
REMOTE_CONF = RMCom.ETC_DIR + "kylin-revision-manager-remote.ini"   # 生效的路径
DISPLAY_CONF = RMCom.ETC_DIR + DISPLY_FILE                          # 生效的路径

ERR_BASE = -1
ERR_FILE_NOT_EXISTS = ERR_BASE - 1
ERR_NOT_OPEN_FILE = ERR_BASE - 2
ERR_NO_KEY_SOURCES = ERR_BASE - 3
ERR_NET_NOT_CONNECTED = ERR_BASE -4
ERR_IN_PARAM = ERR_BASE - 5
ERR_ISO_RETRY = ERR_BASE - 6
ERR_WGET_EXCEPT = ERR_BASE - 7
ERR_ARIA_DIST_EXIST = ERR_BASE - 8

RET_BASE = 1
RET_DOWNLOAD_CANCEL = RET_BASE + 1

SEC_STATE       = 'state'
KEY_PORT        = 'port'
KEY_ISO_PATH    = 'isodownloadpath'
KEY_LICENSE_DIR = 'licenseurl'

#包自带通用0; 服务端获取1; 包定制版本2; 当是2时，从服务端获取到小于等于2的都不使用
DISPLY_JSON_KYLIN = {
    "priority":0,
    "page-active":{
        "display":False,
    },
    "page-advance":{
        "display":False,
        "option-app":{
            "display":False,
            "checked":True
        },
        "option-security":{
            "display":False,
            "checked":True,
            "pwquality":True,
            "userlock":True,
            "exectl":True,
            "devctl":True
        }
    },
    "page-declaration":{
        "display":False,
        "checked":True
    }
}

DISPLY_JSON_OPENKYLIN = {
    "priority":0,
    "page-active":{
        "display":False,
    },
    "page-advance":{
        "display":False,
        "option-app":{
            "display":False,
            "checked":True
        },
        "option-security":{
            "display":False,
            "checked":True,
            "pwquality":True,
            "userlock":True,
            "exectl":True,
            "devctl":True
        }
    },
    "page-declaration":{
        "display":False,
        "checked":True
    }
}


def check_md5(filePath, distMd5):
    if not os.path.exists(filePath):
        logging.info("[RMRemote]: [%s] not exists.", filePath)
        return False

    if len(distMd5) <= 0:
        logging.warning("[RMRemote]: Can not check md5, as no value in sources")
        return True

    try:
        cmd = "md5sum " + filePath
        valueS = get_sys_call(cmd)
        srcMd5 = valueS.split()[0]
        if srcMd5 != distMd5:
            return False
    except Exception as e:
        logging.error("[RMRemote]: Check file md5 err [%s]", e)
        return False
    return True

# 使用nm检测连通性 
def check_nm_connection(index = 0):
    if index > 4:
        return False
    stateI = 0
    try:
        bus = dbus.SystemBus()
        proxy = bus.get_object("org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager")
        bus_interface = dbus.Interface(proxy, dbus_interface="org.freedesktop.DBus.Properties")
        func_getProperity = bus_interface.get_dbus_method("Get")
        if func_getProperity != None:
            stateI = func_getProperity("org.freedesktop.NetworkManager", "State")
        if stateI <= 40:
            index += 1
            sleep(1)
            logging.info("[RMRemote]: [%d] check nm connection state[%d].", index, stateI)
            return check_nm_connection(index)

    except Exception as e:
        logging.error("[RMRemote]: check nm connection err[%s].", e)

    logging.info("[RMRemote]: check nm connection state[%d].", stateI)
    return True

"""curl --http0.9  -X POST -H "Content-Type: application/json" -d 
'{"jsonrpc":"2.0","id":"aria2","method":"aria2.tellStatus","params":["ywx"]}'
http://10.41.120.33:6882/jsonrpc --output curl.log"""
# cmd = "url --http0.9  -X POST -H \"Content-Type: application/json\" -d '{\"jsonrpc\":\"2.0\",\"id\":\"aria2\",\"method\":\"aria2.tellStatus\",\"params\":[\"ywx\"]}' http://10.41.120.33:6882/jsonrpc --output tmp/curl.log"

# reg_sig_func
# set_download_info
# download
# cancel
# _get_free_port
# start_daemon
# is_daemon_exists
# stop_daemon
lock = threading.Lock()
class ARIA2Manger():
    def __init__(self):
        self.btPath = ''            # 种子文件路径
        self.path = ''              # 存放路径
        self.downloadEnd = False    # 下载完成标志
        self.isCancel = False       # 取消标志
        self.func_sig = None        # 进度信号函数
        self.port = None            # rpc端口号
        self.pid = ''               # 后台服务pid
        self.gid = ''               # 下载task的gid
        self.downloadState = 0      # 下载的结果
        self.downloadReason = ''    # 下载的结果说明
        self.lastProcess = 0
        self.lastSpeed = ''
        self.totalLength = 0
        self.completedLength = 0
        self.stopped = False
    
    def get_download_info(self):
        return self.btPath

    def set_download_info(self, btName, distPath):
        if len(distPath) == 0:
            logging.error("[ARIA2M]: distPath is none.")
            return -1
        if len(btName) == 0:
            logging.error("[ARIA2M]: btName is none.")
            return -1

        self.btPath = REMOTE_DOWNLOAD_DIR + btName    # 下载地址
        self.path = distPath    # 存放路径
        return 0

    def _emit_sig(self, process, state = 0, reason='', speedStr='', isoPath=''):
        if self.func_sig != None:
            self.func_sig(state, process, reason, speedStr, isoPath)

    def reg_sig_func(self, func):
        self.func_sig = func

    def _update_process(self, timeOutCount = 0):
        process = 0
        speedOut = ""
        infoDict = {}
        hasProcess = False
        completedLength = self.completedLength
        totalLength = self.totalLength
        if len(self.gid) != 0:
            url = "http://localhost:" + self.port + "/rpc"
            s = client.ServerProxy(url)
            infoDict = s.aria2.tellStatus(self.gid)
            if "errorCode" in infoDict:
                errorCode = infoDict["errorCode"]
                logging.error("[ARIA]: error code [%s] ", errorCode)

                # 12 存在正在下载的
                if errorCode == "13" or errorCode == "12":
                    self.downloadState = ERR_ARIA_DIST_EXIST
                    self.downloadEnd = True
                else:
                    self.downloadState = -1
                    self.downloadEnd = True
                return

            if "downloadSpeed" in infoDict:
                speedStr = infoDict["downloadSpeed"]
                try:
                    speedInt = int(speedStr)
                    if speedInt == 0:
                        timeOutCount += 1
                    else:
                        timeOutCount = 0

                    if timeOutCount > 60 and not check_nm_connection():
                        timeOutCount = 0
                        self.downloadState = ERR_NET_NOT_CONNECTED
                        self.downloadEnd = True
                        logging.error("[ARIA]: Long time no speed.")
                        return

                    if speedInt > 1024 * 1024 *1024:
                        tmpSpeed = speedInt / 1024 /1024 / 1024
                        speedOut = '%.3f'%tmpSpeed + "GB/s"
                    elif speedInt > 1024 * 1024:
                        tmpSpeed = speedInt / 1024 /1024
                        speedOut = '%.3f'%tmpSpeed + "MB/s"
                    elif speedInt > 1024:
                        tmpSpeed = speedInt / 1024
                        speedOut = '%.3f'%tmpSpeed + "KB/s"
                    else:
                        speedOut = '%.3f'%speedInt + "B/s"
                except Exception as e:
                    logging.warning("[ARIA2M]: change [%s] err [%s]", speedStr, e)
                    speedOut = self.lastSpeed

            try:
                if 'completedLength' in infoDict:
                    completedLength =  int(infoDict["completedLength"])
            except Exception as e:
                logging.error("[ARIA]: get completedLength err[%s].", e)

            try:
                if 'totalLength' in infoDict:
                    totalLength = int(infoDict['totalLength'])
            except Exception as e:
                logging.error("[ARIA]: get totalLength err[%s].", e)

            if totalLength > 0:
                self.totalLength = totalLength
                self.completedLength = completedLength
                process = int(completedLength * 100 / totalLength)
                hasProcess = True
            else:
                logging.warning("[ARIA2M]: not get completedLength or totalLength.")

            if hasProcess:
                logging.info("[ARIA2M]: process %d %s", process, speedOut)
                if self.lastProcess < process:
                    self.lastProcess = process
                else:
                    process = self.lastProcess
                self.lastSpeed = speedOut
                if process < 100:
                    self._emit_sig(process, speedStr=speedOut)
                elif process == 100:
                    self._emit_sig(99, speedStr=speedOut)
                    self.downloadEnd = True
                    return
                else:
                    logging.error("[ARIA2M]: Download process[%d] err.", process)
                    return                    
            else:
                logging.warning("[ARIA2M]: not get process info.")

        if self.downloadEnd or self.isCancel:
            logging.info("[ARIA2M]: Download end[%d] or cancel[%d].", self.downloadEnd, self.isCancel)
            return

        threadA = threading.Timer(1, self._update_process, args=[timeOutCount])
        threadA.start()       

    def download(self, index = 0):
        self.start_daemon()
        self.downloadEnd = False
        self.downloadState = 0
        if index > 4:
            self.downloadState = -1
            self.downloadReason = _("unknow error, please call.")
            logging.error("[ARIA2M]: retry time is over.")
            return self.downloadState

        s = None
        try:
            url = "http://localhost:" + self.port + "/rpc"
            s = client.ServerProxy(url)
            opt = {"dir":self.path}
            self.gid = s.aria2.addTorrent(client.Binary(open(self.btPath,'rb').read()), [], opt)

            self._update_process()
        except Exception as e:
            logging.error("[ARIA2M]: add torrent err[%s].", e)
            self.downloadEnd = True
            self.downloadState = -1

        while not self.downloadEnd:
            logging.debug("[ARIA2M]: waiting bt download.")
            sleep(1)

        logging.info("[ARIA2M]: downloadState [%d].", self.downloadState)
        if self.downloadState == 0:
            logging.info("[ARIA2M]: download end.")
        elif self.downloadState == ERR_ARIA_DIST_EXIST:
            self.stop_daemon()
            index = index + 1
            return self.download(index)
        elif self.downloadState == ERR_NET_NOT_CONNECTED:
            self._emit_sig(self.lastProcess, self.downloadState, _("Internet not connected"), self.lastSpeed, '')

        return self.downloadState


    def cancel(self):
        logging.info("[ARIA2M]: start cancel.")
        self.downloadState = RET_DOWNLOAD_CANCEL
        if len(self.gid) == 0:
            logging.warning("[ARIA2M]: cancel no gid.")
            self.isCancel = True
            self.downloadEnd = True
            return 0

        url = "http://localhost:" + self.port + "/rpc"
        try:
            s = client.ServerProxy(url)
            s.aria2.forcePause(self.gid)
            self.isCancel = True
            self.downloadEnd = True
        except Exception as e:
            logging.error("[ARIA2M]: cancel err [%s]",e)
        

    def _get_free_port(self):
        portS = 6800
        portE = 6999
        stateConf = RMConf(REMOTE_STATE_PATH)
        for i in range(portS, portE):
            cmd = "lsof -i:" + str(i) + " | wc -l"
            ret = get_sys_call(cmd)
            if ret.strip("\n") == "0":
                self.port = str(i)
                stateConf.change_key_value(SEC_STATE, KEY_PORT, self.port)
                return
            
    def add_old_exists_download(self):
        path = "/fs.old" + DOWNLOAD_ISO_INFO_PATH
        if not os.path.exists(path):
            logging.info("[ARIA2M]: [%s] not exists.", path)
            return 0
        with open(path, 'r') as fd:
            infoList = fd.readlines()
        fd.close()
        if len(infoList) == 0:
            logging.info("[ARIA2M]: [%s] not info.", path)
            return 0
        for everyline in infoList:
            eleList = everyline.strip("\n").split(":")
            if len(eleList) < 5:
                logging.info("[ARIA2M]: error info [%s].", eleList)
                continue
            if len(eleList) != 6:
                logging.info("[ARIA2M]: This iso not btinfo[%s]",eleList)
                continue

            try:
                btpath = eleList[5]
                oldBtPath = "/fs.old" + btpath
                if not os.path.exists(oldBtPath):
                    logging.info("[ARIA2M]: add old iso [%s] not exists.", oldBtPath)
                    continue
                distDir = eleList[2]
                isoName = eleList[3]
                oldDistDir = "/fs.old" + distDir
                if not os.path.exists(oldDistDir):
                    logging.info("[ARIA2M]: add old iso [%s] not exists",oldDistDir)
                    continue
                oldIsoPath = oldDistDir + isoName
                if not check_md5(oldIsoPath, eleList[4]):
                    logging.error("[ARIA2M]: add iso md5 not same")
                    continue
                url = "http://localhost:" + self.port + "/rpc"
                s = client.ServerProxy(url)
                opt = {"dir":oldDistDir}
                s.aria2.addTorrent(client.Binary(open(oldBtPath,'rb').read()), [], opt)
                logging.info("[ARIA2M]: add torrent [%s].", btpath)
            except Exception as e:
                logging.error("[ARIA2M]: add torrent err[%s].", e)

    def add_exists_download(self):
        if not os.path.exists(DOWNLOAD_ISO_INFO_PATH):
            logging.info("[ARIA2M]: [%s] not exists.", DOWNLOAD_ISO_INFO_PATH)
            return 0
        with open(DOWNLOAD_ISO_INFO_PATH, 'r') as fd:
            infoList = fd.readlines()
        fd.close()
        if len(infoList) == 0:
            logging.info("[ARIA2M]: [%s] not info.", DOWNLOAD_ISO_INFO_PATH)
            return 0
        for everyline in infoList:
            eleList = everyline.strip("\n").split(":")
            if len(eleList) < 5:
                logging.info("[ARIA2M]: error info [%s].", eleList)
                continue
            if len(eleList) != 6:
                logging.info("[ARIA2M]: This iso not btinfo[%s]",eleList)
                continue
            
            try:
                btpath = eleList[5]
                if not os.path.exists(btpath):
                    logging.info("[ARIA2M]: [%s] not exists.", btpath)
                    continue
                distDir = eleList[2]
                isoName = eleList[3]
                if not os.path.exists(distDir):
                    logging.info("[ARIA2M]: add iso [%s] not exists", distDir)
                    continue
                isoPath = distDir + isoName
                if not check_md5(isoPath, eleList[4]):
                    logging.error("[ARIA2M]: add iso md5 not same")
                    continue
                url = "http://localhost:" + self.port + "/rpc"
                s = client.ServerProxy(url)
                opt = {"dir":distDir}
                s.aria2.addTorrent(client.Binary(open(btpath,'rb').read()), [], opt)
                logging.info("[ARIA2M]: add torrent [%s].", btpath)
            except Exception as e:
                logging.error("[ARIA2M]: add torrent err[%s].", e)

    def _accept_aria2c_firewall(self):
        cmd = "iptables -L | grep aria2c | wc -l"
        retStr = get_sys_call(cmd)
        retI = 0
        try:
            retI = int(retStr.strip("\n"))
        except Exception as e:
            logging.error("[ARIA2M]: get iptable info err[%s].", e)
        if retI < 4:
            cmd = "iptables -A KSC_IN_PRIVATE_ALLOW -m comm --comm aria2c -j ACCEPT"
            get_sys_call(cmd)
            cmd = "iptables -A KSC_OUT_PRIVATE_ALLOW -m comm --comm aria2c -j ACCEPT"
            get_sys_call(cmd)
            cmd = "iptables -A KSC_OUT_PUBLIC_ALLOW -m comm --comm aria2c -j ACCEPT"
            get_sys_call(cmd)
            cmd = "iptables -A KSC_IN_PUBLIC_ALLOW -m comm --comm aria2c -j ACCEPT"
            get_sys_call(cmd)

        cmd = "netctl --list pkglist | grep aria2 | wc -l"
        retStr = get_sys_call(cmd)
        if retStr.strip("\n") == "0":
            cmd = "netctl --set -p aria2 -s allow"
            get_sys_call(cmd)

        cmd = "netctl --list applist | grep aria2 | wc -l"
        retStr = get_sys_call(cmd)
        if retStr.strip("\n") == "0":
            cmd = "netctl --set -a /usr/bin/aria2c -s allow"
            get_sys_call(cmd)

    # 加锁处理多线程调用状态可能异常的问题
    def start_daemon(self, index = 0):
        logging.info("[RMRemote] [%d]: start daemon.", index)
        if index == 0:
            self.stopped = False
        else:
            if self.stopped:
                logging.warning("[ARIA2M]: Daemon is stopped.")
                return 0

        lock.acquire()
        if self.is_daemon_exists():
            lock.release()
            return 0

        self._accept_aria2c_firewall()

        self._get_free_port()
        cmd = "aria2c -c -D --enable-rpc --bt-enable-lpd=true --log-level=notice --log=/var/log/RevisionsManager/aria.log --rpc-listen-port=" + self.port + " --allow-overwrite --seed-ratio=0.0 --check-integrity"
        ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode != 0:
            logging.error("[ARIA2M]: [%s] return[%d] out[%s] err[%s].", cmd, ret.returncode, ret.stdout.decode().strip("\n"), ret.stderr.decode().strip("\n"))
            lock.release()
            return -1

        if index < 12:
            sleep(5)
        else:
            sleep(120)

        try:
            cmd = "ps -lax | grep \"aria2c -c -D --enable-rpc --bt-enable-lpd=true\" | grep -v \"ps -lax\" | grep rpc-listen-port=  |  awk \'{print $3}\'"
            retStr = get_sys_call(cmd)
            if len(retStr) == 0:
                index = index + 1
                timerA = threading.Timer(2, self.start_daemon, args=[index])
                timerA.start()
                lock.release()
                return -1
            infoList = retStr.strip("\n").split("\n")
            if len(infoList) == 1:
                self.pid = retStr.strip("\n")
            else:
                self.pid = infoList[0]
                logging.warning("[ARIA2M]: Exist other daemon[%s].", infoList)
        except Exception as e:
            logging.error("[ARIA2M]: Err [%s]", e)

        self.add_exists_download()

        self.add_old_exists_download()

        self._check_logFile()
        lock.release()
        return 0

    def is_daemon_exists(self):
        try:
            stateConf = RMConf(REMOTE_STATE_PATH)
            self.port = stateConf.get_key(SEC_STATE, KEY_PORT)
        except Exception as e:
            logging.error("[ARIA2M]: get port err[%s]", e)

        if self.port == None or len(self.port) == 0:
            logging.warning("[ARIA2M]: no port info.")
            return False

        cmd = "ps -lax | grep \"aria2c -c -D --enable-rpc --bt-enable-lpd=true\" | grep -v \"ps -lax\" | grep rpc-listen-port=" + self.port + " |  awk \'{print $3}\'"
        retStr = get_sys_call(cmd)
        if len(retStr) == 0:
            logging.info("[ARIA2M]: Daemon not exists.")
            return False
        infoList = retStr.strip("\n").split("\n")
        if len(infoList) == 1:
            self.pid = retStr.strip("\n")
        else:
            self.pid = infoList[0]
            logging.warning("[ARIA2M]: Exist other daemon %s.", infoList)

        logging.info("[ARIA2M]: Exist daemon %s.", infoList)
        return True

    def stop_daemon(self, index = 0):
        if index > 5:
            logging.error("[ARIA2M]: Stop daemon failed.")
            return False

        if not self.is_daemon_exists():
            self.stopped = True
            logging.warning("[ARIA2M]: daemon not exists.")
            self.pid = ''
            return True

        cmd = "kill " + self.pid
        get_sys_call(cmd)
        index += 1
        sleep(1)
        return self.stop_daemon(index)


    def _get_file_size(self, filePath):
        fsize = os.path.getsize(filePath)
        fsize_M = fsize / float(1024 * 1024)
        return round(fsize_M, 2)

    def _check_logFile(self):
        try:
            logFilePath1 = "/var/log/RevisionsManager/aria.log"
            if not os.path.exists(logFilePath1):
                return
            fileSize = self._get_file_size(logFilePath1)
            if fileSize < 300:
                return
            logFilePath2 = "/var/log/RevisionsManager/aria.log.1"
            if os.path.exists(logFilePath2):
                os.remove(logFilePath2)
            logging.info("[ARIA2M]: Change log name.")
            os.rename(logFilePath1, logFilePath2)
        except Exception as e:
            logging.error("[ARIA2M]: Change log name err[%s].", e)

        timerA = threading.Timer(3600, self._check_logFile)
        timerA.start()
        return

# set_download_info
# reg_sig_func
# download_times
# download
# cancel

class WGETManger():
    RETRY_COUNT = 5
    def __init__(self):
        self.url = ''               # 下载地址
        self.path = ''              # 存放路径
        self.downloadEnd = False    # 下载完成标志
        self.isCancel = False       # 取消标志
        self.func_sig = None        # 进度信号函数
        self.logPath = LOG_PATH
        self.isShow = False
        self.lastProcess = 0
        self.lastSpeed = ''
        self.downloadState = 0

    def get_download_info(self):
        return self.url

    def set_download_info(self, url, distPath, flag):
        if len(distPath) == 0:
            logging.error("[WGETM]: distPath is none.")
            return -1
        if len(url) == 0:
            logging.error("[WGETM]: url is none.")
            return -1
        if len(flag) == 0:
            logging.error("[WGETM]: flag is none.")
            return -1
        self.url = url          # 下载地址
        self.path = distPath    # 存放路径
        self.logPath = LOG_PATH + "_" + flag
        return 0

    def reg_sig_func(self, func):
        self.func_sig = func

    def _emit_sig(self, process, state = 0, reason='', speedStr='', isoPath=''):
        if self.func_sig != None and self.isShow:
            self.func_sig(state, process, reason, speedStr, isoPath)

    def _update_process(self):
        process = 0
        speedStr = ""
        hasProcess = False
        lineList = []
        if not os.path.exists(self.logPath):
            logging.warning("[WGETM]: Process info [%s] not exists.", self.logPath)
        else:
            try:
                with open(self.logPath, 'r') as fd:
                    lineList = fd.readlines()
                fd.close()
                linelen = len(lineList)
                for i in range(1, 6):
                    if i > linelen:
                        logging.info("[WGETM]: Process info line %d is over %d.", i, linelen)
                        break;
                    infoList = lineList[-i].split()
                    if "The file is already fully retrieved; nothing to do." in infoList or "文件已下载完成；不会进行任何操作。" in infoList:
                        self.downloadState = 0
                        self._emit_sig(99, reason=_("File exists"))
                        logging.info("[WGETM]: File exists. Nothing to do.")
                        return
                    elif "Giving up." in infoList or "放弃操作。" in infoList:
                        self.downloadState = -1
                        self._emit_sig(self.lastProcess, state=-1, reason=_("Internet not connected"))
                        return
                    elif "无法解析主机地址" in infoList:
                        self.downloadState = -1
                        self._emit_sig(self.lastProcess, state=-1, reason=_("Server not connected"))
                        return

                    if len(infoList) < 4:
                        continue 

                    if "错误" in infoList or "ERROR" in infoList:
                        logging.error("[WGETM]: Process Download err[%s].", lineList[-i].strip("\n"))
                        self.downloadState = -1
                        self._emit_sig(self.lastProcess, state=-1, reason=_("Download err"))
                        return
                    elif "100%" in infoList:
                        process = 100
                        speedStr = infoList[-1].split("=")[0] + "/s"
                        break
                    elif "已保存" in infoList or "saved" in infoList:
                        return
                    else:
                        if infoList[-3].find("%") == -1:
                            continue
                        speedStr = infoList[-2] + "/s"
                        processStr = infoList[-3].strip("%")
                        process = int(processStr)
                        hasProcess = True
                        break
                if hasProcess:
                    logging.info("[WGETM]: process %d %s", process, speedStr)
                    if self.lastProcess < process:
                        self.lastProcess = process
                    self.lastSpeed = speedStr
                    if process < 100:
                        self._emit_sig(process, speedStr=speedStr)
                    elif process == 100:
                        self._emit_sig(99, speedStr=speedStr)
                        logging.info("[WGETM]: Download is 100%.")
                        return
                    else:
                        logging.error("[WGETM]: Download process[%d] err.", process)
                        return                    
                else:
                    logging.warning("[WGETM]: not get process info.")
            except Exception as e:
                logging.error("[WGETM]: update process err[%s].",e)

        if self.downloadEnd or self.isCancel:
            logging.info("[WGETM]: Download end or cancel.")
            return
        threadA = threading.Timer(1, self._update_process)
        threadA.start()

    def download_times(self, isShow = False):
        index = 0
        retI = 0
        reason = ''
        while index < 5:
            retI = self.download(isShow)
            if retI == 0 or retI == RET_DOWNLOAD_CANCEL:
                break
            sleep(1)
            index = index +1

        # 错误信息处理
        if retI != 0 and retI != RET_DOWNLOAD_CANCEL:
            logging.error("[WGETM]: Download wget return[%d].", retI)
            if retI == 4:
                reason = _("Internet not connected")
            elif retI == 5 or retI == 8:
                reason = _("Server not connected")
            elif retI == ERR_WGET_EXCEPT:
                reason = _("Wget break down!")
            else:
                reason = _("Can not download , please reboot and try again.")
            self._emit_sig(self.lastProcess, state=-1, reason=reason, speedStr = self.lastSpeed)

        return retI, reason

    def download(self, isShow = False):
        if len(self.url) == 0:
            logging.error("[WGETM]: download no url.")
            return -1
        if len(self.path) == 0:
            logging.error("[WGETM]: download no distpath.")
            return -1
        logging.info("[WGETM]: start [%s] to [%s].", self.url, self.path)
        self.downloadEnd = False
        self.isCancel = False
        self.isShow = isShow
        if isShow:
            thread_w = threading.Timer(1, self._update_process)
            thread_w.start()

        out = None
        err = None
        try:
            """wget -c -P /var/cache/RevisionsManager/download/ 
            http://builder.kylin.com/kybuilder/download/raw/upgrade-test/52316/Kylin-Desktop-V10-SP1-General-beta4-Build01-2403-arm64.iso 
            -o /var/cache/RevisionsManager/download/wget_log  --progress=bar:force"""
            cmdList = ['wget', '-c', '-P', self.path, self.url, '-o', self.logPath, '-T', '5', '--progress=dot']
            self.isoSP = subprocess.Popen(cmdList)
            out, err = self.isoSP.communicate()

            self.downloadEnd = True
            if self.isCancel:
                logging.debug("[WGETM]: Is canceled, return.")
                return RET_DOWNLOAD_CANCEL
        except Exception as e:
            logging.error("[WGETM]: Download err[%s].", e)
            self.downloadEnd = True
            self.isoSP = None
            return ERR_WGET_EXCEPT

        ret = self.isoSP.returncode
        self.isoSP = None
        if ret != 0:
            if out != None:
                logging.error("[WGETM]: Download wget out[%s].", out.decode())
            if err != None:
                logging.error("[WGETM]: Download wget err[%s].", err.decode())
        logging.info("[WGETM]: Download end.")
        return ret

    def cancel(self):
        if self.isoSP == None:
            logging.info("[WGETM]: Download not exists.")
            return 0, ''
        else:
            self.isoSP.terminate()
        self.isoSP = None
        self.isCancel = True
        self.downloadEnd = True
        logging.info("[WGETM]: Download is canceled.")
        return 0, ''


class RMRemote(RMCom):
    SEC_SPECREMOTE          = "SpecRemote"
    SEC_REMOTE              = "Remote"
    REMOTE_KEY_ADDRESS      = "address"
    SEC_SERVICE             = "servicekey"
    KEY_KEY                 = "key"
    REMOTE_KEY_SOURCES      = "sources"
    REMOTE_KEY_SERVICEKEY   = "servicekey"
    REMOTE_KEY_ARCH         = "arch"
    REMOTE_KEY_MAC          = "mac"
    REMOTE_KEY_STATE        = "state"
    REMOTE_KEY_ISOPATH      = "isopath"
    REMOTE_KEY_ISOSIZE      = "isosize"
    REMOTE_KEY_ISOMD5       = "isomd5"
    REMOTE_KEY_CFGPATH      = "cfgpath"
    REMOTE_KEY_NAME         = "name"
    REMOTE_KEY_VERSION      = "version"
    REMOTE_KEY_BTPATH       = "btpath"
    REMOTE_KEY_ACTIVATION   = "activation"
    REMOTE_KEY_ACTIVATEPATH = "activatepath"

    def __init__(self):
        self.url = DEFAULT_REMOTE_ADDRESS       # 实际使用的地址
        self.oldUrl = DEFAULT_REMOTE_ADDRESS    # 旧系统源地址
        self.specAddress = ''                   # 指定源地址
        self.address = self.url                 # 默认源地址
        self.conf_json = DISPLY_JSON_KYLIN
        self.servicekey = ""                    # 服务序列号
        self.osInfoList = []                    # 与服务序列号匹配的 OS 信息
        self.osOutList = []                     # 输出的信息 给界面
        self.activePath = ''                    # 授权文件下载地址
        self.checkStep = [0, 5, 20, 30, 100]
        self.arch = get_sys_call("uname -m").strip("\n")
        self.checkProcess = None
        self.downloadProcess = None
        self.runningHD = None
        self.btSw = True
        self.lastCheckProcess = 0

        self.osInfo = RMOsInfo()
        if not os.path.exists(REMOTE_DOWNLOAD_DIR):
            os.makedirs(REMOTE_DOWNLOAD_DIR, mode=0o755)
        self.get_remote_address()
        self.servicekey = self._get_service_key()
        self.get_download_format()

        if self.btSw:
            ARIA2HD = ARIA2Manger()
            tmpThread = threading.Thread(target=ARIA2HD.start_daemon)
            tmpThread.start()

    # 获取服务序列号
    def _get_service_key(self, isOld = False):
        ret = ''
        distPath = KYINFO_PATH
        if isOld:
            distPath = "/fs.old" + KYINFO_PATH

        if not os.path.exists(distPath):
            logging.warning("[RMRemote]: [%s] not exists.", distPath)
            if self.osInfo.osNum == self.osInfo.VERSION_OPENKYLIN:
                ret = 'openkylin'
            elif self.osInfo.osNum == self.osInfo.VERSION_UKYLIN:
                ret = 'ubuntukylin'
            elif self.osInfo.osNum == self.osInfo.VERSION_KYLIN_V10:
                ret = 'kylinV10'
            elif self.osInfo.osNum == self.osInfo.VERSION_KYLIN_V10SP1:
                ret = 'kylinV10SP1'
            else:
                ret = 'kylinV10'
        else:
            try:
                kyinfo = RMConf(distPath)
                ret = kyinfo.get_key(self.SEC_SERVICE, self.KEY_KEY)
            except Exception as e:
                logging.error("[RMRemote]: read [%s] err [%s].", distPath, e)
        logging.debug("[RMRemote]: Get service key is [%s].", ret)
        return ret

    # 将映射表表中的地址转换为绝对路径
    def _convert_download_path(self, inPath):
        if len(inPath) == 0:
            return ""
        if inPath.startswith("http"):
            return inPath
        if inPath.startswith("/"):
            infoListA = self.url.split("://")
            finalPath = infoListA[0] + "://" + infoListA[1].split("/")[0] + inPath
            return finalPath
        else:
            return self.url + inPath

    def set_remote_address(self, address):
        try:
            remote_cfg = RMConf(REMOTE_CONF)
            remote_cfg.change_key_value(self.SEC_SPECREMOTE, self.REMOTE_KEY_ADDRESS, address)
            self.url = address
            if not self.url.endswith("/"):
                self.url = self.url + "/"
        except Exception as e:
            logging.error("[RMRemote]: read [%s] err [%s].", REMOTE_CONF, e)


    # 获取服务端地址
    def get_remote_address(self):
        if not os.path.exists(REMOTE_CONF):
            logging.warning("[RMRemote]: [%s] not exists.", REMOTE_CONF)
        else:
            try:
                remote_cfg = RMConf(REMOTE_CONF)
                self.specAddress = remote_cfg.get_key(self.SEC_SPECREMOTE, self.REMOTE_KEY_ADDRESS)
                self.address = remote_cfg.get_key(self.SEC_REMOTE, self.REMOTE_KEY_ADDRESS)
                logging.info("[RMRemote]: get remote address [%s].", self.specAddress)
                return self.specAddress
            except Exception as e:
                logging.error("[RMRemote]: read [%s] err [%s].", REMOTE_CONF, e)

        logging.error("[RMRemote]: there is no address in [%s].", REMOTE_CONF)
        return ""
    
    def get_remote_default_address(self):
        if not os.path.exists(REMOTE_CONF):
            logging.warning("[RMRemote]: [%s] not exists.", REMOTE_CONF)
        else:
            try:
                remote_cfg = RMConf(REMOTE_CONF)
                self.specAddress = remote_cfg.get_key(self.SEC_SPECREMOTE, self.REMOTE_KEY_ADDRESS)
                self.address = remote_cfg.get_key(self.SEC_REMOTE, self.REMOTE_KEY_ADDRESS)
                logging.info("[RMRemote]: get default remote address [%s].", self.address)
                return self.address
            except Exception as e:
                logging.error("[RMRemote]: read [%s] err [%s].", REMOTE_CONF, e)

        return self.address

    # 设置BT开关
    def set_download_format(self, sw):
        ariaHd = ARIA2Manger()
        if sw:
            swStr = "True"
            # 切换后立即点击下载 可能有风险
            # 不起线程 会卡主界面
            tmpThread = threading.Thread(target=ariaHd.start_daemon)
            tmpThread.start()
        else:
            swStr = "False"
            tmpThread = threading.Thread(target=ariaHd.stop_daemon)
            tmpThread.start()

        etcConf = RMConf(self.CONF_PATH)
        etcConf.change_key_value(etcConf.SEC_SWITCH, etcConf.KEY_SW_BT, swStr)
        self.btSw = sw
        logging.info("[RMRemote]: Set used bt [%d].",sw)

    # 获取BT开关
    def get_download_format(self):
        if not os.path.exists(self.CONF_PATH):
            logging.info("[RMRemote]: Get used bt True.")
            self.btSw = True
            return True
        etcConf = RMConf(self.CONF_PATH)
        swStr = etcConf.get_key(etcConf.SEC_SWITCH, etcConf.KEY_SW_BT)
        if swStr != None and swStr == 'False':
            self.btSw = False
            logging.info("[RMRemote]: Get used bt False.")
            return False

        self.btSw = True
        logging.info("[RMRemote]: Get used bt True.")
        return True

    # 服务端地址
    def _get_url(self):
        self.get_remote_address()
        if self.specAddress != None and self.specAddress != "":
            self.url = self.specAddress
        elif self.address != None and self.address != "":
            self.url = self.address

        if not self.url.endswith("/"):
            self.url = self.url + "/"

        logging.info("[RMRemote] remote address is [%s].", self.url)

    # 授权文件下载
    def _download_license(self, activeUrl, distDir = REMOTE_DOWNLOAD_DIR):
        if len(activeUrl) == 0:
            logging.error("[RMRemote] activePath is none.")
            return

        if os.path.exists(distDir + LICENSE_FILE):
            os.remove(distDir + LICENSE_FILE)
        src = activeUrl + LICENSE_FILE
        wgetHDA = WGETManger()
        wgetHDA.set_download_info(src, distDir, "L")
        wgetHDA.download_times()

        if os.path.exists(distDir + KYINFO_FILE):
            os.remove(distDir + KYINFO_FILE)
        src = activeUrl + KYINFO_FILE
        wgetHDB = WGETManger()
        wgetHDB.set_download_info(src, distDir, "k")
        wgetHDB.download_times()

    # 授权文件下载
    def download_license(self):
        if len(self.activePath) == 0:
            logging.error("[RMRemote] activePath is none.")
            return

        if not os.path.exists(DL_LICENSE_PATH):
            src = self.activePath + LICENSE_FILE
            wgetHDA = WGETManger()
            wgetHDA.set_download_info(src, REMOTE_DOWNLOAD_DIR, "L")
            wgetHDA.download_times()

        if not os.path.exists(DL_KYINFO_PATH):
            src = self.activePath + KYINFO_FILE
            wgetHDB = WGETManger()
            wgetHDB.set_download_info(src, REMOTE_DOWNLOAD_DIR, "k")
            wgetHDB.download_times()

    # 记录下载的镜像信息
    def _record_iso_info(self, info, isoMd5):
        infoList = []
        if os.path.exists(DOWNLOAD_ISO_INFO_PATH):
            with open(DOWNLOAD_ISO_INFO_PATH, 'r') as fd:
                infoList = fd.readlines()
            fd.close()
        if len(infoList) != 0:
            for everyline in infoList:
                eleList = everyline.strip("\n").split(":")
                if len(eleList) >= 5 and eleList[4] == isoMd5:
                    logging.info("[RMRemote] This iso info[%s] exists.", isoMd5)
                    return
        try:
            with open(DOWNLOAD_ISO_INFO_PATH, 'a') as fd:
                fd.write(info)
            fd.close()
        except Exception as e:
            logging.error("[RMRemote] write download iso info err[%s].", e)

    # 根据镜像信息获取 下载地址
    def _get_path_from_isoInfo(self, name, version):
        path = ""
        if not os.path.exists(DOWNLOAD_ISO_INFO_PATH):
            return path
        with open(DOWNLOAD_ISO_INFO_PATH, 'r') as fd:
            infoList = fd.readlines()
        fd.close()
        if len(infoList) == 0:
            return path
        for everyline in infoList:
            eleList = everyline.strip("\n").split(":")
            if len(eleList) >= 5 and eleList[0] == name and eleList[1] == version:
                path = eleList[2] + eleList[3]
                logging.info("[RMRemote] This iso[%s][%s] path is [%s].",name, version, path)
                return path

        logging.error("[RMRemote] not find [%s] [%s]", name, version)
        return path

    # 下载source 进度回调
    def _show_download_sources_process(self, state, process, reason, speedStr, isoPath):
        try:
            newprocess = int(process * (self.checkStep[2] - self.checkStep[1]) / 100) + self.checkStep[1]
            self.show_check_process(state, newprocess)
        except Exception as e:
            logging.error("[RMRemote]: show check process err[%s].", e)
        

    # 下载源信息
    def _download_sources(self, isold = False):
        src = self.url + SOURCES_FILE
        distDir = REMOTE_DOWNLOAD_DIR
        if isold:
            distDir = "/fs.old" + REMOTE_DOWNLOAD_DIR
            src = self.oldUrl + SOURCES_FILE

        wgetHD = WGETManger()
        wgetHD.reg_sig_func(self._show_download_sources_process)
        wgetHD.set_download_info(src, distDir, "s")
        return wgetHD.download_times(True) 

     # 下载显示配置
    def _download_display(self, cfgPath):
        if os.path.exists(DL_DISPLAY_PATH):
            os.remove(DL_DISPLAY_PATH)

        wgetHD = WGETManger()
        wgetHD.set_download_info(cfgPath, REMOTE_DOWNLOAD_DIR, "d")
        wgetHD.download_times()

        priority_n = 0
        priority = 0
        if not os.path.exists(DL_DISPLAY_PATH):
            logging.error("[RMRemote] [%s] not exists.", DL_DISPLAY_PATH)
            return

        try:
            with open(DL_DISPLAY_PATH, 'r') as fd:
                infoJson = json.load(fd)
            fd.close()
            priority_n = int(infoJson["priority"])
        except Exception as e:
            logging.error("[RMRemote] read [%s] err [%s].", DL_DISPLAY_PATH, e)

        if not os.path.exists(DISPLAY_CONF):
            cmd = "cp " + DL_DISPLAY_PATH + " " + self.ETC_DIR
            get_sys_call(cmd)
            return

        try:
            with open(DISPLAY_CONF, 'r') as fd:
                infoJson = json.load(fd)
            fd.close()
            priority = infoJson["priority"]
        except Exception as e:
            logging.error("[RMRemote] read [%s] err [%s].", DISPLAY_CONF, e)

        if priority_n > priority:
            cmd = "cp " + DL_DISPLAY_PATH + " " + self.ETC_DIR
            get_sys_call(cmd)

    # 源信息解析
    def _analysis_sources(self):
        if not os.path.exists(DL_SOURCES_PATH):
            logging.error("[RMRemote] [%s] not exists.", DL_SOURCES_PATH)
            self.show_check_process(ERR_FILE_NOT_EXISTS, self.checkStep[3] + 5, _("Sources file not exists."))
            return ERR_FILE_NOT_EXISTS

        infoJson = {}
        try:
            with open(DL_SOURCES_PATH, 'r') as fd:        
                infoJson = json.load(fd)
            fd.close()
        except Exception as e:
            cmd = "rm -f " + DL_SOURCES_PATH
            get_sys_call(cmd)
            logging.error("[RMRemote] load [%s] err [%s].", DL_SOURCES_PATH, e)
            self.show_check_process(ERR_NOT_OPEN_FILE, self.checkStep[3] + 5, _("Load sources file error."))
            return ERR_NOT_OPEN_FILE
        
        if self.REMOTE_KEY_SOURCES not in infoJson.keys():
            logging.error("[RMRemote] [%s] not in json", self.REMOTE_KEY_SOURCES)
            self.show_check_process(ERR_NO_KEY_SOURCES, self.checkStep[3] + 5, _("Sources info not in json."))
            return ERR_NO_KEY_SOURCES

        sourcesList = infoJson[self.REMOTE_KEY_SOURCES]
        self.osOutList = []
        count = len(sourcesList)
        if count > 0:
            for i in range(0, count):
                try:
                    ele = sourcesList[i]
                    isSameArch = False
                    for everyArch in self.ARCHLIST:
                        if self.arch in everyArch and ele[self.REMOTE_KEY_ARCH] in everyArch:
                            isSameArch = True
                            break

                    if isSameArch and self.servicekey == ele[self.REMOTE_KEY_SERVICEKEY]:
                        if len(ele[self.REMOTE_KEY_ISOPATH]) > 0:
                            self.osInfoList.append(ele)
                            infoStr = ele[self.REMOTE_KEY_NAME] + ":" + ele[self.REMOTE_KEY_VERSION] + ":" + str(ele[self.REMOTE_KEY_ISOSIZE]) + "G"
                            self.osOutList.append(infoStr)
                            # 目前找到一个就退出 
                            break
                        else:
                            logging.warning("[RMRemote] [%s] [%s] has not iso path.", ele[self.REMOTE_KEY_ARCH], ele[self.REMOTE_KEY_SERVICEKEY])
                except Exception as e:
                    logging.error("[RMRemote] analysis sources err [%s]", e)

                process = int(self.checkStep[3] - 1 + (i+1) * (90 - self.checkStep[3]) / count)
                self.show_check_process(0, process)

        self.show_check_process(0, 90, "", self.osOutList)

        if self.REMOTE_KEY_ACTIVATION not in infoJson.keys():
            logging.error("[RMRemote] [%s] not in json", self.REMOTE_KEY_ACTIVATION)
            return 0

        activeList = infoJson[self.REMOTE_KEY_ACTIVATION]
        for ele in activeList:
            try:
                isSameArch = False
                for everyArch in self.ARCHLIST:
                    if self.arch in everyArch and ele[self.REMOTE_KEY_ARCH] in everyArch:
                        isSameArch = True
                        break
                if isSameArch and self.servicekey == ele[self.REMOTE_KEY_SERVICEKEY]:
                    if len(ele[self.REMOTE_KEY_ACTIVATEPATH]) > 0:
                        self.activePath = self._convert_download_path(ele[self.REMOTE_KEY_ACTIVATEPATH])
                        if not self.activePath.endswith("/"):
                            self.activePath = self.activePath + "/"
                        # 目前找到一个就退出
                        break
                    else:
                        logging.warning("[RMRemote] [%s] [%s] has not activate file Path.", ele[self.REMOTE_KEY_ARCH], ele[self.REMOTE_KEY_SERVICEKEY])
                    break
            except Exception as e:
                logging.error("[RMRemote] analysis activate info err [%s]", e)
        return 0

    # 源信息解析
    def _get_old_activateUrl(self, serviceKey):
        distPath = "/fs.old" + DL_SOURCES_PATH
        if not os.path.exists(distPath):
            logging.error("[RMRemote] [%s] not exists.", distPath)
            return ''

        infoJson = {}
        try:
            with open(distPath, 'r') as fd:        
                infoJson = json.load(fd)
            fd.close()
        except Exception as e:
            logging.error("[RMRemote] load [%s] err [%s].", distPath, e)
            return ''

        if self.REMOTE_KEY_ACTIVATION not in infoJson.keys():
            logging.error("[RMRemote] [%s] not in json", self.REMOTE_KEY_ACTIVATION)
            return ''

        activeList = infoJson[self.REMOTE_KEY_ACTIVATION]
        for ele in activeList:
            try:
                isSameArch = False
                for everyArch in self.ARCHLIST:
                    if self.arch in everyArch and ele[self.REMOTE_KEY_ARCH] in everyArch:
                        isSameArch = True
                        break
                if isSameArch and serviceKey == ele[self.REMOTE_KEY_SERVICEKEY]:
                    if len(ele[self.REMOTE_KEY_ACTIVATEPATH]) > 0:
                        activePath = self._convert_download_path(ele[self.REMOTE_KEY_ACTIVATEPATH])
                        if not activePath.endswith("/"):
                            activePath = activePath + "/"
                        return activePath
                    else:
                        logging.warning("[RMRemote] [%s] [%s] has not activate file Path.", ele[self.REMOTE_KEY_ARCH], ele[self.REMOTE_KEY_SERVICEKEY])                    
            except Exception as e:
                logging.error("[RMRemote] analysis activate info err [%s]", e)
        return ''


    def _get_old_url(self):
        distPath = "/fs.old" + REMOTE_CONF
        if not os.path.exists(distPath):
            logging.warning("[RMRemote]: [%s] not exists.", distPath)
            return

        try:
            remote_cfg = RMConf(distPath)
            specAddress = remote_cfg.get_key(self.SEC_SPECREMOTE, self.REMOTE_KEY_ADDRESS)
            address = remote_cfg.get_key(self.SEC_REMOTE, self.REMOTE_KEY_ADDRESS)
            logging.info("[RMRemote]: Get Old remote address [%s][%s].", specAddress, address)
            if specAddress != None and specAddress != "":
                self.oldUrl = specAddress
            elif address != None and address != "":
                self.oldUrl = address
        except Exception as e:
            logging.error("[RMRemote]: read [%s] err [%s].", distPath, e)

        if not self.oldUrl.endswith("/"):
            self.oldUrl = self.oldUrl + "/"

        logging.info("[RMRemote] Old remote address is [%s].", self.oldUrl)

    def get_license_from_sources_only(self, isOld = False):
        prefixPath = ""
        if isOld:
            prefixPath = "/fs.old"
        downloadDir = prefixPath + REMOTE_DOWNLOAD_DIR
        fileL = prefixPath + DL_LICENSE_PATH
        fileK = prefixPath + DL_KYINFO_PATH
        if os.path.exists(fileL) and os.path.exists(fileK):
            logging.info("[RMRemote] Both exist in [%s]", downloadDir)
            return

        serviceKey = self._get_service_key(isOld)
        if isOld:
            self._get_old_url()
        else:
            self._get_url()
        self._check_connection()
        ret, reason = self._download_sources(isOld)
        if ret != 0:
            logging.error("[RMRemote]:  download sources failed [%s].", reason)
            return
        if isOld:
            oldActivePath = self._get_old_activateUrl(serviceKey)
            self._download_license(oldActivePath, distDir=downloadDir)
        else:
            self._analysis_sources()
            self._download_license(self.activePath)
        pass

    # 检测更新
    def get_sources_info(self):
        self.servicekey = self._get_service_key()
        self._get_url()
        self.lastCheckProcess = self.checkStep[0]
        self.show_check_process(0, self.checkStep[0])
        ret = check_nm_connection()
        if not ret:
            self.show_check_process(-1, self.checkStep[1], _("Internet not connected"))
            return
        self._check_connection()

        self.show_check_process(0, self.checkStep[1])

        ret, reason = self._download_sources()
        if ret != 0:
            self.show_check_process(-1, self.checkStep[2], reason)
            return
        self.show_check_process(0, self.checkStep[2])

        self.show_check_process(0, self.checkStep[3])
        self._analysis_sources()
        self.show_check_process(0, self.checkStep[4], "", self.osOutList)
        self._download_license()
    
    # "1.1.1.1" -> [0x1,0x1,0x1,0x1]
    def _change_strList_to_intList(self, inList):
        outList = [0]*len(inList)
        try:
            for i in range(len(inList)):
                outList[i] =  int(inList[i])
        except Exception as e:
            logging.error("[RMRemote]: change err[%s].", e)
            return []
        return outList

    def _is_same_group(self, ipA, maskA, ipB):
        if len(ipA) == 0 or len(maskA) == 0 or len(ipB) == 0:
            return False
        ipAIntList = self._change_strList_to_intList(ipA.split("."))
        maskAIntList = self._change_strList_to_intList(maskA.split("."))
        ipBIntList = self._change_strList_to_intList(ipB.split("."))
        if len(ipAIntList) != len(ipBIntList) or len(ipAIntList) != len(maskAIntList):
            logging.warning("[RMRemote]: info not right.", ipAIntList, maskAIntList, ipBIntList)
            return False
        for i in range(len(maskAIntList)):
            tmpA = maskAIntList[i] & ipAIntList[i]
            tmpB = maskAIntList[i] & ipBIntList[i]
            if tmpA != tmpB:
                return False
        return True


    # 服务端联通性检测
    def _check_connection(self):
        nameList = self.url.split("://")
        if len(nameList) == 2:
            pathA = nameList[1]
            pathMain = pathA.split("/")[0]
            if ":" in pathMain:
                name = pathMain.split(":")[0]
            else:
                name = pathMain
        else:
            logging.error("[RMRemote]: Address[%s] is err.", self.url)
            return False
        cmd = "ping " + name + " -c 2"
        ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode == 0:
            logging.info("[RMRemote]: Connection is good.")
            return True
        elif ret.returncode == 1:
            infoStr = ret.stdout.decode().split("\n")[0]
            tmpIp = infoStr.split()[2]
            finalIpStr = tmpIp[1:-1]
            finalIpList = finalIpStr.split(".")
            if len(finalIpList) != 4:
                logging.error("[RMRemote]: Get ip[%s] err.", finalIpStr)
                return False
        elif ret.returncode == 2:
            logging.error("[RMRemote]: Check connection ret[%d] out[%s] err[%s].", ret.returncode, ret.stdout.decode(), ret.stderr.decode())
            return False
        else:
            logging.error("[RMRemote]: Check connection ret[%d] out[%s] err[%s].", ret.returncode, ret.stdout.decode(), ret.stderr.decode())
            return False

        cmd = "ls /sys/devices/virtual/net/"
        ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode != 0:
            logging.error("[RMRemote]: find virtual net ret[%d] out[%s] err[%s].", ret.returncode, ret.stdout.decode(), ret.stderr.decode())
            return False

        virtualNetList = ret.stdout.decode().split("\n")
        for every in virtualNetList:
            if len(every) < 1:
                continue
            cmd = "ifconfig " + every + " | grep inet | grep -v inet6"
            ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if ret.returncode != 0:
                logging.error("[RMRemote]: [%s] ret[%d] out[%s] err[%s].", cmd, ret.returncode, ret.stdout.decode(), ret.stderr.decode())
                continue

            pattern = re.compile(r'(([1-9]?\d|1\d\d|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d\d|2[0-4]\d|25[0-5])')
            infoList = ret.stdout.decode().split()
            ipStr = ''
            maskStr = ''
            if ":" in ret.stdout.decode() == 1:
                for i in range(len(infoList)):
                    if infoList[i] == "inet":
                        ipStr = infoList[i+1]
                    elif infoList[i] == "netmask":
                        maskStr = infoList[i+1]
            else:
                for i in range(len(infoList)):
                    if "addr" in infoList[i] or "地址" in infoList[i]:
                        tmpIp = pattern.search(infoList[i])
                        if tmpIp == None:
                            logging.warning("[RMRemote]: There is no ip in %s.", infoList[i])
                            continue
                        ipStr = infoList[i][tmpIp.start():tmpIp.end()]
                    elif "Mask" in infoList[i] or "掩码" in infoList[i]:
                        tmpMask = pattern.search(infoList[i])
                        if tmpMask == None:
                            logging.warning("[RMRemote]: There is no mask in %s.", infoList[i])
                            continue
                        maskStr = infoList[i][tmpMask.start():tmpMask.end()]

            logging.info("[RMRemote]: check info [%s] [%s] [%s].", ipStr, maskStr, finalIpStr)
            if self._is_same_group(ipStr, maskStr, finalIpStr):
                cmd = "ifconfig " + every + " down"
                logging.info("[RMRemote]: act [%s].", cmd)
                ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                if ret.returncode != 0:
                    logging.error("[RMRemote]: [%s] ret[%d] out[%s] err[%s].", cmd, ret.returncode, ret.stdout.decode(), ret.stderr.decode())
                    continue
            else:
                logging.info("[RMRemote]: check not same.")

        cmd = "ping " + name + " -c 2"
        ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode == 0:
            logging.info("[RMRemote]: Connection is good.")
            return True
        else:
            logging.info("[RMRemote]: Connection is not good ret[%d] out[%s] err[%s].", cmd, ret.returncode, ret.stdout.decode(), ret.stderr.decode())
            return False

    # 获取默认配置路径
    def get_base_display(self):
        if not os.path.exists(DISPLAY_CONF):
            try:
                if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                    jsonStr = json.dump(DISPLY_JSON_KYLIN)
                else:
                    jsonStr = json.dump(DISPLY_JSON_OPENKYLIN)
                with open(DISPLAY_CONF, "w")as fd:
                    fd.write(jsonStr)
                fd.close()
            except Exception as e:
                logging.error("[RMRemote] get base display err[%s].", e)
        return DISPLAY_CONF

    # 检测镜像是否下载完成
    def check_iso_exist(self, name, version):
        infoList = []
        if not os.path.exists(DOWNLOAD_ISO_INFO_PATH):
            logging.debug("[RMRemote] file [%s] not exists", DOWNLOAD_ISO_INFO_PATH)
            return False, ''

        with open(DOWNLOAD_ISO_INFO_PATH, 'r') as fd:
            infoList = fd.readlines()
        fd.close()
        if len(infoList) == 0:
            return False, ''

        for everyline in infoList:
            try:
                eleList = everyline.strip("\n").split(":")
                tmpName = eleList[0]
                tmpVer = eleList[1]
                tmpPath = eleList[2]
                tmpFile = eleList[3]
                tmpMd5 = eleList[4]
                if tmpName == name and tmpVer == version and len(tmpMd5) == 32:
                    distIsoPath = tmpPath + tmpFile
                    if check_md5(distIsoPath, tmpMd5):
                        return True, distIsoPath
            except Exception as e:
                logging.warning("[RMRemote] get download iso info err[%s].", e)
        return False, ''

    def show_iso_process(self, state, process, reason='', speedStr='', isoPath=''):
        if self.downloadProcess != None:
            self.downloadProcess(state, process, reason, speedStr, isoPath)

    def show_check_process(self, state, process, reason='', retList=[]):
        # logging.debug("[RMRemote]: state[%d] process[%d] [%s].", state, process, reason)
        if self.lastCheckProcess >= process:
            return
        self.lastCheckProcess = process
        if self.checkProcess != None:
            self.checkProcess(state, process, reason, retList)

    # 检测进度信号
    def reg_check_process_func(self, func):
        self.checkProcess = func
    
    # 下载进度信号
    def reg_download_process_func(self, func):
        self.downloadProcess = func

    # 重新下载
    def retry_download(self, name, version):
        logging.debug("[RMRemote]: Retry download iso.")
        path = self._get_path_from_isoInfo(name, version)
        path_part = path + ".part"
        if os.path.exists(path):
            logging.info("[RMRemote]: remove old file[%s].", path)
            os.remove(path)
        if os.path.exists(path_part):
            logging.info("[RMRemote]: remove old file[%s].", path_part)
            os.remove(path_part)

        self.start_download(name, version)

    
    def _get_downloadInfo_from_isoInfo(self, name, version):
        cfgPath = ''
        isoPath = ''
        isoMd5 = ''
        isoSize = 0
        btPath = ''

        for ele in self.osInfoList:
            if self.REMOTE_KEY_SERVICEKEY not in ele.keys() or self.servicekey != ele[self.REMOTE_KEY_SERVICEKEY]:
                logging.info("[RMRemote] key [%s] is not.", self.servicekey)
                continue
            if self.REMOTE_KEY_NAME not in ele.keys() or name != ele[self.REMOTE_KEY_NAME]:
                logging.info("[RMRemote] name [%s] is not.", name)
                continue
            if self.REMOTE_KEY_VERSION not in ele.keys() or version != ele[self.REMOTE_KEY_VERSION]:
                logging.info("[RMRemote] version [%s] is not.", version)
                continue
            if self.REMOTE_KEY_ISOPATH not in ele.keys():
                logging.info("[RMRemote] key [%s] not exists.", self.REMOTE_KEY_ISOPATH )
                continue

            isoMd5 = ele[self.REMOTE_KEY_ISOMD5]
            isoPath = self._convert_download_path(ele[self.REMOTE_KEY_ISOPATH])
            if self.REMOTE_KEY_BTPATH in ele:
                btPath = self._convert_download_path(ele[self.REMOTE_KEY_BTPATH])

            if self.REMOTE_KEY_CFGPATH in ele.keys():
                cfgPath = self._convert_download_path(ele[self.REMOTE_KEY_CFGPATH])
            try:
                isoSize = float(ele[self.REMOTE_KEY_ISOSIZE]) * self.SPACE_G
            except Exception as e:
                logging.error("[RMRemote] can not get iso size [%s].", e)
                isoSize = 5 * self.SPACE_G
            
            # 预防配置错误
            if isoSize < 3 * self.SPACE_G:
                isoSize = 5 * self.SPACE_G

            logging.debug("[RMRemote] Find iso info [%s %s %s %F].", ele[self.REMOTE_KEY_SERVICEKEY], ele[self.REMOTE_KEY_NAME], ele[self.REMOTE_KEY_VERSION], isoSize)
            break

        return isoPath, btPath, isoMd5, isoSize, cfgPath

    def download_iso(self, srcAddress, distPath, md5Value, btName = ''):
        logging.info("[RMRemote]: Download iso start [%d][%s].", self.btSw, btName)
        isoFullName = srcAddress.split("/")[-1]
        distIsoPath = distPath + isoFullName
        ret = 0

        self.show_iso_process(0, 0)
        ret = check_md5(distIsoPath, md5Value)
        if ret:
            self.show_iso_process(0, 100, isoPath=distIsoPath)
            return 0
        
        if self.runningHD != None:
            logging.info("[RMRemote]: Download is running [%s].", self.runningHD.get_download_info())
            return

        if self.btSw and len(btName) > 0:
            ariaHD = ARIA2Manger()
            self.runningHD = ariaHD
            ariaHD.reg_sig_func(self.downloadProcess)
            ariaHD.set_download_info(btName, distPath)
            ret = ariaHD.download()
        else:
            wgetHD = WGETManger()
            self.runningHD = wgetHD
            wgetHD.reg_sig_func(self.downloadProcess)
            wgetHD.set_download_info(srcAddress, distPath, "iw")
            ret, tmpS = wgetHD.download_times(isShow=True)

        # TODO: 下载状态处理 reason赋值
        self.runningHD = None
        reason = ''

        if ret == RET_DOWNLOAD_CANCEL:
            logging.debug("[RMRemote]: Download iso is canceled, return.")
            return RET_DOWNLOAD_CANCEL

        if ret != 0:
            self.show_iso_process(ret, self.lastProcess, reason)
            logging.error("[RMRemote]: Download iso ret[%d] reason[%s].", ret, reason)
            return ret

        if not check_md5(distIsoPath, md5Value):
            logging.error("[RMRemote]: download [%s] md5 cfg [%s].", distIsoPath, md5Value)
            self.show_iso_process(ERR_ISO_RETRY, self.lastProcess, _("Md5sum is err"))
            return ERR_ISO_RETRY

        self.show_iso_process(0, 100, isoPath=distIsoPath)
        logging.info("[RMRemote]: Download iso end.")
        return 0


    # 开始下载
    def start_download(self, name, version):
        if len(name) <= 0 or len(version) <= 0:
            logging.error("[RMRemote] name[%s] version[%s].", name, version)
            return ERR_IN_PARAM, _("Name or version is err")

        self.show_iso_process(0, 0)

        isoPath = ''
        btPath = ''
        isoMd5 = ''
        isoSize = 0
        cfgPath = ''
        btName = ''
        try:
            isoPath,btPath,isoMd5,isoSize,cfgPath  = self._get_downloadInfo_from_isoInfo(name, version)
        except Exception as e:
            logging.error("[RMRemote] start download err[%s].", e)
            self.show_iso_process(-1, 0, _("Get iso info error."))

        if len(cfgPath) > 0:
            cfgThread = threading.Thread(target=self._download_display, args=[cfgPath])
            cfgThread.start()

        if len(btPath) > 0 and self.btSw:
            btName = btPath.split("/")[-1]
            wgetHD = WGETManger()
            wgetHD.set_download_info(btPath, REMOTE_DOWNLOAD_DIR, "b")
            wgetHD.download_times()

        distPath = get_download_path(isoSize)
        if distPath == None:
            logging.error("[RMRemote] can not find enough space")
            self.show_iso_process(-1, 0, _("There is not enough space"))
            return -1, _("There is not enough space")

        if len(isoPath) == 0:
            logging.warning("[RMRemote] Can not find iso.")
            self.show_iso_process(-1, 0, _("Can not find iso"))
            return -1, _("Can not find iso")
        
        if len(btName) > 0 and self.btSw:
            info = name + ":" + version + ":" + distPath + ":" + isoPath.split("/")[-1]  + ":" + isoMd5 + ":" + REMOTE_DOWNLOAD_DIR + btName + "\n"
        else:
            info = name + ":" + version + ":" + distPath + ":" + isoPath.split("/")[-1]  + ":" + isoMd5 + "\n"
        self._record_iso_info(info, isoMd5)

        try:
            self.lastProcess = 0
            isoThread = threading.Thread(target=self.download_iso, args=[isoPath, distPath, isoMd5, btName])
            isoThread.start()
        except Exception as e:
            logging.error("[RMRemote] start download err[%s].", e)
            self.show_iso_process(-1, self.lastProcess, _("Download error, please retry."))
            return -1, _("Download error, please retry.")

        return 0, ''

    # 取消下载
    def cancel_download(self):
        if self.runningHD != None:
            self.runningHD.cancel()
            self.runningHD = None
        logging.info("[RMRemote]: Download is canceled.")
        return 0, ''
    
    def isdownloading(self):
        if self.runningHD != None:
            logging.info("[RMRemote]: Is downloading %s.", self.osOutList)
            return True,self.osOutList
        else:
            return False,[]
    

def test_process_sig(process, state = 0, reason='', speedStr='', distPath=''):
    logging.info("test [%d] [%d] [%s] [%s] [%s]",process, state, reason, speedStr, distPath)

def test_wget():
    # set_download_info
    # reg_sig_func
    # download_times
    # download
    # cancel
    isoPath = "http://distro-images.kylinos.cn:8802/web_pungi/download/share/yzxYCtBN4lOZXs761vPmq2hfwEWFRLK5/Kylin-Desktop-V10-SP1-General-Release-2303-ARM64.iso"
    url = "https://distro-images.kylinos.cn:8802/web_pungi/download/share/upgrade-config/"
    Surl = url + SOURCES_FILE
    obgWget = WGETManger()
    obgWget.set_download_info(Surl, REMOTE_DOWNLOAD_DIR, "s")
    obgWget.download_times()
    sleep(1)
    obgWget.cancel()

def test_aria():
    # reg_sig_func
    # set_download_info
    # download
    # cancel
    # _get_free_port
    # start_daemon
    # is_daemon_exists
    # stop_daemon

    # btpath = "http://distro-images.kylinos.cn:8802/web_pungi/download/share/yzxYCtBN4lOZXs761vPmq2hfwEWFRLK5/Kylin-Desktop-V10-SP1-General-Release-2303-ARM64.iso.torrent"
    btpath = "/home/ywx/tmp/Kylin-Desktop-V10-SP1-General-Release-2303-X86_64.iso.torrent"
    objAria = ARIA2Manger()
    # md5 = "7b36c8d49f7453565e9a015f2bcbcf2b"
    # objAria._get_free_port()
    objAria.reg_sig_func(test_process_sig)
    objAria.set_download_info(btpath, "/.tmpiso")
    objAria.start_daemon()
    objAria.download()

    pass

def test_remote():
    obj = RMRemote()
    # srcPath = "http://172.20.191.70/v10-upgrade/kylin-revision-manager-sources.json"
    srcPath = "http://builder.kylin.com/kybuilder/download/raw/upgrade-test/52316/Kylin-Desktop-V10-SP1-General-beta4-Build01-2403-arm64.iso"
    obj.get_sources_info()
    pass   


if __name__ == "__main__":
    loglevel = logging.DEBUG
    FORMAT = "%(asctime)s [%(levelname)s]: %(message)s"
    logFileName = "/var/log/RevisionsManager/RevisionsManager.log.1"
    logging.basicConfig(format=FORMAT, level=loglevel, datefmt='%m-%d,%H:%M:%S', filename=logFileName, filemode='a')

    test_remote()
    # test_wget()
    # test_aria()

